# Import this module from a running iPython or Jupyter notebook
# to load the magic functions:
#
# %%savefig_dark_light figname
#
# %%savefig_dark figname
#
# %%savefig_light figname

from IPython.core import magic_arguments
from IPython.core.magic import (line_magic, cell_magic, Magics, magics_class)
from sys import stderr
import matplotlib.pyplot as plt


@magics_class
class FigureMagics(Magics):
    
    darkstyle = 'easy-dark'
    lightstyle = 'easy-light'
    darkfigdir = ''
    lightfigdir = ''
    darkappend = '_dark'
    
    @line_magic
    def set_darkstyle(self, line):
        self.darkstyle = line

    @line_magic
    def get_darkstyle(self, line):
        return self.darkstyle
        
    @line_magic
    def set_lightstyle(self, line):
        self.lightstyle = line

    @line_magic
    def get_lightstyle(self, line):
        return self.lightstyle

    @line_magic
    def set_darkfigdir(self, line):
        self.darkfigdir = line

    @line_magic
    def get_darkfigdir(self, line):
        return self.darkfigdir
        
    @line_magic
    def set_lightfigdir(self, line):
        self.lightfigdir = line

    @line_magic
    def get_lightfigdir(self, line):
        return self.lightfigdir

    @line_magic
    def set_darkappend(self, line):
        self.dark_append = line
    
    @line_magic
    def get_darkappend(self, line):
        return self.darkappend
        
    @cell_magic
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('figname', help='Base Figure Name to save to')
    def savefig_dark_light(self, line='', cell=None):
        """
        Saves both a light-background and dark-background version of the
        Matplotlib figure generated in the enclosed cell.

        Arguments
        ---------
        figname : name of the figure to save
            Dark/Light figure will be saved in two seperate directories
        """
        args = magic_arguments.parse_argstring(self.savefig_dark_light, line)
        light_figname = self.lightfigdir + args.figname
        if '.' in args.figname:
            pre, _, post = args.figname.rpartition('.')
            dark_figname = self.darkfigdir + pre + self.darkappend + '.' + post
        else:
            dark_figname = self.darkfigdir + args.figname + self.darkappend
        with plt.style.context((self.lightstyle)):
            self.shell.user_ns['is_dark'] = False
            self.shell.ex(cell)
            stderr.write('Saving light figure as %s\n' % (light_figname))
            plt.savefig(light_figname)
        with plt.style.context((self.darkstyle)):
            self.shell.user_ns['is_dark'] = True
            self.shell.ex(cell)
            stderr.write('Saving dark figure as %s\n' % (dark_figname))
            plt.savefig(dark_figname)

    @cell_magic
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('figname', help='Base Figure Name to save to')
    def savefig_dark(self, line='', cell=None):
        """
        Saves a dark-background version of the
        Matplotlib figure generated in the enclosed cell.

        Arguments
        ---------
        figname : name of the figure to save
            Will be saved in directory for dark figures
        """
        args = magic_arguments.parse_argstring(self.savefig_dark, line)
        if '.' in args.figname:
            pre, _, post = args.figname.rpartition('.')
            dark_figname = self.darkfigdir + pre + self.darkappend + '.' + post
        else:
            dark_figname = self.darkfigdir + args.figname + self.darkappend
        with plt.style.context((self.darkstyle)):
            self.shell.user_ns['is_dark'] = True
            self.shell.ex(cell)
            stderr.write('Saving dark figure as %s\n' % (dark_figname))
            plt.savefig(dark_figname)
            
    @cell_magic
    @magic_arguments.magic_arguments()
    @magic_arguments.argument('figname', help='Base Figure Name to save to')
    def savefig_light(self, line='', cell=None):
        """
        Saves a light-background version of the
        Matplotlib figure generated in the enclosed cell.

        Arguments
        ---------
        figname : name of the figure to save
            Will be saved in directory for light figures
        """
        args = magic_arguments.parse_argstring(self.savefig_light, line)
        light_figname = self.lightfigdir + args.figname
        with plt.style.context((self.lightstyle)):
            self.shell.user_ns['is_dark'] = False
            self.shell.ex(cell)
            stderr.write('Saving light figure as %s\n' % (light_figname))
            plt.savefig(light_figname)

            
# This loads the functions above into your running iPython instance
ip = get_ipython()
ip.register_magics(FigureMagics)



# -*- coding: utf-8 -*-

import os
import re
import codecs
from setuptools import setup, find_packages, Command
from setuptools.command.install import install
import matplotlib as mpl
import glob
import os.path
import shutil
import sys


here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


class CustomInstall(install):
    """
    Also copy over the matplotlib files
    """
    def run(self):
        install.run(self)

    
# https://packaging.python.org/guides/single-sourcing-package-version/
def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


with open(os.path.join(here, 'README.md'), 'r') as f:
    long_description = f.read()


setup(
    name="easydarkfigs",
    version=find_version('easydarkfigs', '__init__.py'),
    install_requires=['matplotlib', 'IPython'],
    packages=find_packages(),
    license='LICENSE',
    author="Ben Cook",
    author_email="bacook17@gmail.com",
    description="iPython Magic for Easy Dark/Light Figures",
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords="matplotlib, dark figures",
    url="https://github.com/bacook17/easydarkfigs/",
    package_data={'easydarkfigs': ['mplstyles/*.mplstyle']},
    cmdclass={'install': CustomInstall},
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering :: Visualization",
    ]
)

# https://stackoverflow.com/questions/35851201/how-can-i-share-matplotlib-style
BASE_LIBRARY_PATH = os.path.join(mpl.get_configdir(), 'stylelib')
STYLE_PATH = os.path.join(here, 'easydarkfigs/mplstyles/')
STYLE_EXTENSION = 'mplstyle'
style_files = glob.glob(os.path.join(STYLE_PATH, "*.%s" % (STYLE_EXTENSION)))
for _path_file in style_files:
    _, fname = os.path.split(_path_file)
    dest = os.path.join(BASE_LIBRARY_PATH, fname)
    if not os.path.isfile(dest):
        shutil.copy(_path_file, dest)
        print("%s style installed" % (fname))
    else:
        print("%s style already exists" % (fname))
